$(function() {
    var $tbody = $('tbody'),
        $thead = $('thead'),
        $table = $('table'),
        $nav = $('nav'),
        $edit = $('#edit'),
        $help = $('#help'),
        $helpButton = $('#help-button'),
        $boldButton = $('#bold-button'),
        $italicButton = $('#italic-button'),
        $underlineButton = $('#underline-button'),
        $fontSelect = $('#font-select'),
        $colorSelect = $('#color-select'),
        $bgSelect = $('#bg-select'),
        $bgRgb = $('#bg-rgb'),
        $fontRgb = $('#font-rgb'),
        $removeFormatting = $('#remove-formatting'),
        $removeColumn = $('#remove-column'),
        $removeRow = $('#remove-row'),
        $plus = $('#plus'),
        $minus = $('#minus'),
        $selected = null,
        fonts = ['times', 'arial', 'verdana', 'courier', 'comic'],
        colors = ['font-black', 'font-blue', 'font-red', 'font-yellow', 'font-green'],
        bgColors = ['bg-white', 'bg-blue', 'bg-red', 'bg-yellow', 'bg-green'];

    // initial page setup
    $table.css('margin-top', $nav.height());

    // generate column labels
    // IIFE kvůli tomu že kvůli kompatibilitě nepoužívám block scoped let a const, mohlo by to být i v samostatné funkci mimo...
    (function() {
        var $tr = $('tr', $thead);

        for (i = 0; i < 26; i++) {
            var $td = $('<td class="label">' + String.fromCharCode(i + 65) + '</td>');
            $td.appendTo($tr);
        }
    })();

    // generate row labels
    (function() {
        for (i = 1; i < 101; i++) {
            $('<tr><td class="label">' + i + '</td></tr>').appendTo($tbody);
        }
    })();

    // generate cells, pokud by jich byl statický počet vždy, tak by bylo lepší vygenerovat pomocí scriptu přímo html soubor...
    (function () {
        $('tr', $tbody).each(function(i) {
            for (j = 0; j < 26; j++) {
                $(this).append($('<td />'));
            }
        });
    })();

    // on cell click
    $tbody.click(function(e) {
        var $target = $(e.target);
        if (!$target.hasClass('label') && $target.is('td')) {
            if ($selected) $selected.removeClass('selected');
            $selected = $target;
            $selected.addClass('selected');

            $edit.val($selected.html());
            $edit.focus();

            $fontSelect.val('times');
            fonts.forEach(function(e) {
                if ($selected.hasClass(e)) $fontSelect.val(e);
            });

            $colorSelect.val('font-black');
            colors.forEach(function(e) {
                if ($selected.hasClass(e)) $colorSelect.val(e);
            });

            $bgSelect.val('bg-white');
            bgColors.forEach(function(e) {
                if ($selected.hasClass(e)) $bgSelect.val(e);
            });

            $bgRgb.val($selected[0].style.backgroundColor);

            $fontRgb.val($selected[0].style.color);
        }
    });

    $edit.on('keypress change', function() {
        var self = $(this);
        if ($selected) $selected.html(self.val());
    })

    // extensions
    $helpButton.click(function() {
        $help.toggle();
    });

    $boldButton.click(function() {
        if ($selected) $selected.toggleClass('bold');
    });

    $italicButton.click(function() {
        if ($selected) $selected.toggleClass('italic');
    });

    $underlineButton.click(function() {
        if ($selected) $selected.toggleClass('underline');
    });

    $fontSelect.change(function() {
        var $option = $('option:selected', $(this));
        if ($selected) {
            $selected.removeClass(fonts.join(' '));
            $selected.addClass($option.val());
        }
    });

    $colorSelect.change(function() {
        var $option = $('option:selected', $(this));
        if ($selected) {
            $selected.removeClass(colors.join(' '));
            $selected.addClass($option.val());
            $selected.css('color', '');
        }
    });

    $bgSelect.change(function() {
        var $option = $('option:selected', $(this));
        if ($selected) {
            $selected.removeClass(bgColors.join(' '));
            $selected.addClass($option.val());
            $selected.css('background-color', '');
        }
    });

    $bgRgb.on('keypress change', function() {
        if ($selected) {
            $selected.css('background-color', $(this).val());
            $selected.removeClass(bgColors.join(' '));
        }
    });

    $fontRgb.on('keypress change', function() {
        if ($selected) {
            $selected.css('color', $(this).val());
            $selected.removeClass(colors.join(' '));
        }
    });

    $plus.click(function() {
        if ($selected) {
            var size = $selected[0].style.fontSize || '14px';
            size = Number(size.substring(0, size.length - 2));
            $selected.css('font-size', (size + 2) + 'px');
        }
    });

    $minus.click(function() {
        if ($selected) {
            var size = $selected[0].style.fontSize || '14px';
            size = Number(size.substring(0, size.length - 2));
            $selected.css('font-size', (size - 2) + 'px');
        }
    });

    $removeFormatting.click(function() {
        if ($selected) {
            $selected.removeClass();
            $selected.attr('style', '');
            $selected.addClass('selected');
        }
    });

    $removeColumn.click(function() {
        if ($selected) {
            $('tbody tr td:nth-child(' + Number($selected.index()+1) + ')').remove();
            $('tbody tr').each(function() {
                $(this).append('<td />');
            });
            $selected = null;
        }
    });

    $removeRow.click(function() {
        if ($selected) {
            var index = Number($selected.parent('tr').index() + 1);
            $('tbody tr:nth-child(' + index + ')').remove();
            $('tbody .label').each(function() {
                var self = $(this);
                if (index < Number(self.html())) {
                    self.html(Number(self.html() - 1));
                }
            });

            var tr = $('<tr/>').appendTo($tbody)
            tr.append('<td class="label">100</td>')
            for (i = 0; i < 26; i++) {
                tr.append($('<td />'));
            }
            $selected = null;
        }
    });

    $(document).keydown(function(e) {
        if ($selected) {
            switch (e.which) {
                case 37: //doleva
                	var target = $selected.prev();
                    target.trigger('click');
                    break;
                case 38: //nahoru
                    var column = $selected.index() + 1;
                    var row = $selected.parent('tr').index();
                    var target = $('tbody tr:nth-child(' + row + ') td:nth-child(' + column + ')');
                    target.trigger('click');
                    break;
                case 39: //doprava
                    var target = $selected.next();
                    target.trigger('click');
                    break;
                case 40: //dolů
                    var column = $selected.index() + 1;
                    var row = $selected.parent('tr').index() + 2;
                    var target = $('tbody tr:nth-child(' + row + ') td:nth-child(' + column + ')');
                    target.trigger('click');
                    break;
                case 13: //enter
                    if ($edit.is(':focus')) {
                        var column = $selected.index() + 1;
                        var row = $selected.parent('tr').index() + 2;
                        var target = $('tbody tr:nth-child(' + row + ') td:nth-child(' + column + ')');
                        target.trigger('click');
                    } else {
                        $edit.focus();
                    }
                    break;
                case 9: //tab
                    var target = $selected.next();
                    target.trigger('click');
                    break;
                default:
                    return;
            }
            e.preventDefault();
        }
    });
});
